@@ -1,37 +1,42 @@
 // Copyright 2020-2021 the Deno authors. All rights reserved. MIT license.
-use crate::swc_util::AstParser;
-use crate::ReexportKind;
-use futures::future::LocalBoxFuture;
-use swc_common::comments::CommentKind;
-use swc_common::Span;
-use swc_ecmascript::ast::Decl;
-use swc_ecmascript::ast::DefaultDecl;
-use swc_ecmascript::ast::ExportSpecifier;
-use swc_ecmascript::ast::Expr;
-use swc_ecmascript::ast::ImportSpecifier;
-use swc_ecmascript::ast::ModuleDecl;
-use swc_ecmascript::ast::ModuleItem;
-use swc_ecmascript::ast::Stmt;
-use swc_ecmascript::parser::Syntax;
 
+use crate::js_doc::JsDoc;
 use crate::namespace::NamespaceDef;
 use crate::node;
 use crate::node::DocNode;
 use crate::node::ModuleDoc;
-use crate::swc_util;
+use crate::swc_util::get_location;
+use crate::swc_util::js_doc_for_span;
 use crate::ImportDef;
 use crate::Location;
-use futures::FutureExt;
-use regex::Regex;
+use crate::ReexportKind;
+
+use deno_ast::swc::ast::Decl;
+use deno_ast::swc::ast::DefaultDecl;
+use deno_ast::swc::ast::ExportSpecifier;
+use deno_ast::swc::ast::Expr;
+use deno_ast::swc::ast::ImportSpecifier;
+use deno_ast::swc::ast::ModuleDecl;
+use deno_ast::swc::ast::ModuleItem;
+use deno_ast::swc::ast::Stmt;
+use deno_ast::ParsedSource;
+use deno_graph::MediaType;
+use deno_graph::ModuleGraph;
+use deno_graph::ModuleSpecifier;
+use deno_graph::Resolved;
+use deno_graph::SourceParser;
+
 use std::collections::HashMap;
 use std::error::Error;
 use std::fmt;
+use std::sync::Arc;
 
 #[derive(Debug)]
 pub enum DocError {
   Resolve(String),
+  #[allow(dead_code)]
   Io(std::io::Error),
-  Parse(swc_util::SwcDiagnosticBuffer),
+  Parse(deno_ast::Diagnostic),
 }
 
 impl Error for DocError {}
@@ -47,25 +52,12 @@ impl fmt::Display for DocError {
   }
 }
 
-impl From<swc_util::SwcDiagnosticBuffer> for DocError {
-  fn from(error: swc_util::SwcDiagnosticBuffer) -> DocError {
+impl From<deno_ast::Diagnostic> for DocError {
+  fn from(error: deno_ast::Diagnostic) -> DocError {
     DocError::Parse(error)
   }
 }
 
-pub trait DocFileLoader {
-  fn resolve(
-    &self,
-    specifier: &str,
-    referrer: &str,
-  ) -> Result<String, DocError>;
-
-  fn load_source_code(
-    &self,
-    specifier: &str,
-  ) -> LocalBoxFuture<Result<(Syntax, String), DocError>>;
-}
-
 #[derive(Clone)]
 enum ImportKind {
   Namespace(String),
@@ -78,17 +70,21 @@ struct Import {
   kind: ImportKind,
 }
 
-pub struct DocParser {
-  pub ast_parser: AstParser,
-  pub loader: Box<dyn DocFileLoader>,
+pub struct DocParser<'a> {
+  pub ast_parser: &'a dyn SourceParser,
+  pub graph: ModuleGraph,
   pub private: bool,
 }
 
-impl DocParser {
-  pub fn new(loader: Box<dyn DocFileLoader>, private: bool) -> Self {
+impl<'a> DocParser<'a> {
+  pub fn new(
+    graph: ModuleGraph,
+    private: bool,
+    ast_parser: &'a dyn SourceParser,
+  ) -> Self {
     DocParser {
-      loader,
-      ast_parser: AstParser::default(),
+      ast_parser,
+      graph,
       private,
     }
   }
@@ -97,19 +93,24 @@ impl DocParser {
   /// as well as a list of reexported items which need to be fetched from other modules.
   pub fn parse_module(
     &self,
-    file_name: &str,
-    syntax: Syntax,
-    source_code: &str,
+    specifier: &ModuleSpecifier,
+    media_type: MediaType,
+    source_code: Arc<String>,
   ) -> Result<ModuleDoc, DocError> {
-    let parse_result =
-      self.ast_parser.parse_module(file_name, syntax, source_code);
-    let module = parse_result?;
+    let parsed_source =
+      self
+        .ast_parser
+        .parse_module(specifier, source_code, media_type)?;
+    let module = parsed_source.module();
     let mut doc_entries =
-      self.get_doc_nodes_for_module_body(module.body.clone());
-    let import_doc_entries =
-      self.get_doc_nodes_for_module_imports(module.body.clone(), file_name)?;
+      self.get_doc_nodes_for_module_body(&parsed_source, module.body.clone());
+    let import_doc_entries = self.get_doc_nodes_for_module_imports(
+      &parsed_source,
+      module.body.clone(),
+      specifier,
+    )?;
     doc_entries.extend(import_doc_entries);
-    let reexports = self.get_reexports_for_module_body(module.body);
+    let reexports = self.get_reexports_for_module_body(&module.body);
     let module_doc = ModuleDoc {
       definitions: doc_entries,
       reexports,
@@ -118,27 +119,46 @@ impl DocParser {
   }
 
   /// Fetches `file_name` and parses it.
-  pub async fn parse(&self, file_name: &str) -> Result<Vec<DocNode>, DocError> {
-    let (syntax, source_code) = self.loader.load_source_code(file_name).await?;
+  #[cfg(feature = "rust")]
+  pub fn parse(
+    &self,
+    specifier: &ModuleSpecifier,
+  ) -> Result<Vec<DocNode>, DocError> {
+    let module = self
+      .graph
+      .try_get(specifier)
+      .map_err(|err| DocError::Resolve(err.to_string()))?
+      .ok_or_else(|| {
+        DocError::Resolve(format!(
+          "Unable to load specifier: \"{}\"",
+          specifier
+        ))
+      })?;
 
-    self.parse_source(file_name, syntax, source_code.as_str())
+    self.parse_source(
+      &module.specifier,
+      module.media_type,
+      module.source.clone(),
+    )
   }
 
   /// Parses a module and returns a list of exported items (no reexports).
+  #[cfg(feature = "rust")]
   pub fn parse_source(
     &self,
-    file_name: &str,
-    syntax: Syntax,
-    source_code: &str,
+    specifier: &ModuleSpecifier,
+    media_type: MediaType,
+    source_code: Arc<String>,
   ) -> Result<Vec<DocNode>, DocError> {
-    let module_doc = self.parse_module(file_name, syntax, source_code)?;
-    Ok(module_doc.definitions)
+    self
+      .parse_module(specifier, media_type, source_code)
+      .map(|md| md.definitions)
   }
 
-  async fn flatten_reexports(
+  fn flatten_reexports(
     &self,
     reexports: &[node::Reexport],
-    referrer: &str,
+    referrer: &ModuleSpecifier,
   ) -> Result<Vec<DocNode>, DocError> {
     let mut by_src: HashMap<String, Vec<node::Reexport>> = HashMap::new();
 
@@ -154,8 +174,11 @@ impl DocParser {
     }
 
     for specifier in by_src.keys() {
-      let resolved_specifier = self.loader.resolve(specifier, referrer)?;
-      let doc_nodes = self.parse_with_reexports(&resolved_specifier).await?;
+      let resolved_specifier = self
+        .graph
+        .resolve_dependency(specifier, referrer)
+        .ok_or_else(|| DocError::Resolve(specifier.clone()))?;
+      let doc_nodes = self.parse_with_reexports(resolved_specifier)?;
       let reexports_for_specifier = by_src.get(specifier).unwrap();
 
       for reexport in reexports_for_specifier {
@@ -174,7 +197,7 @@ impl DocParser {
                 line: 1,
                 col: 0,
               },
-              None,
+              JsDoc::default(),
               ns_def,
             );
             processed_reexports.push(ns_doc_node);
@@ -210,46 +233,73 @@ impl DocParser {
   }
 
   /// Fetches `file_name`, parses it, and resolves its reexports.
-  pub fn parse_with_reexports<'a>(
-    &'a self,
-    file_name: &'a str,
-  ) -> LocalBoxFuture<Result<Vec<DocNode>, DocError>> {
-    async move {
-      let (syntax, source_code) =
-        self.loader.load_source_code(file_name).await?;
+  pub fn parse_with_reexports(
+    &self,
+    specifier: &ModuleSpecifier,
+  ) -> Result<Vec<DocNode>, DocError> {
+    let module = self
+      .graph
+      .try_get(specifier)
+      .map_err(|err| DocError::Resolve(err.to_string()))?
+      .ok_or_else(|| {
+        DocError::Resolve(format!(
+          "Unable to load specifier: \"{}\"",
+          specifier
+        ))
+      })?;
 
-      let module_doc = self.parse_module(file_name, syntax, &source_code)?;
+    let module = if let Some((_, Resolved::Specifier(types_specifier, _))) =
+      &module.maybe_types_dependency
+    {
+      self
+        .graph
+        .try_get(types_specifier)
+        .map_err(|err| DocError::Resolve(err.to_string()))?
+        .ok_or_else(|| {
+          DocError::Resolve(format!(
+            "Unable to load specifier: \"{}\"",
+            specifier
+          ))
+        })?
+    } else {
+      module
+    };
 
-      let flattened_docs = if !module_doc.reexports.is_empty() {
-        let mut flattenned_reexports = self
-          .flatten_reexports(&module_doc.reexports, file_name)
-          .await?;
-        flattenned_reexports.extend(module_doc.definitions);
-        flattenned_reexports
-      } else {
-        module_doc.definitions
-      };
+    let module_doc = self.parse_module(
+      &module.specifier,
+      module.media_type,
+      module.source.clone(),
+    )?;
 
-      Ok(flattened_docs)
-    }
-    .boxed_local()
+    let flattened_docs = if !module_doc.reexports.is_empty() {
+      let mut flattenned_reexports =
+        self.flatten_reexports(&module_doc.reexports, &module.specifier)?;
+      flattenned_reexports.extend(module_doc.definitions);
+      flattenned_reexports
+    } else {
+      module_doc.definitions
+    };
+
+    Ok(flattened_docs)
   }
 
   fn get_doc_nodes_for_module_imports(
     &self,
-    module_body: Vec<swc_ecmascript::ast::ModuleItem>,
-    referrer: &str,
+    parsed_source: &ParsedSource,
+    module_body: Vec<deno_ast::swc::ast::ModuleItem>,
+    referrer: &ModuleSpecifier,
   ) -> Result<Vec<DocNode>, DocError> {
     let mut imports = vec![];
 
     for node in module_body.iter() {
-      if let swc_ecmascript::ast::ModuleItem::ModuleDecl(ModuleDecl::Import(
+      if let deno_ast::swc::ast::ModuleItem::ModuleDecl(ModuleDecl::Import(
         import_decl,
       )) = node
       {
-        let (js_doc, location) = self.details_for_span(import_decl.span);
+        let js_doc = js_doc_for_span(parsed_source, &import_decl.span);
+        let location = get_location(parsed_source, import_decl.span.lo);
         for specifier in &import_decl.specifiers {
-          use swc_ecmascript::ast::ImportSpecifier::*;
+          use deno_ast::swc::ast::ImportSpecifier::*;
 
           let (name, maybe_imported_name, src) = match specifier {
             Named(named_specifier) => (
@@ -273,9 +323,12 @@ impl DocParser {
             ),
           };
 
-          let resolved_specifier = self.loader.resolve(&src, referrer)?;
+          let resolved_specifier = self
+            .graph
+            .resolve_dependency(&src, referrer)
+            .ok_or_else(|| DocError::Resolve(src.clone()))?;
           let import_def = ImportDef {
-            src: resolved_specifier,
+            src: resolved_specifier.to_string(),
             imported: maybe_imported_name,
           };
 
@@ -292,29 +345,33 @@ impl DocParser {
 
   pub fn get_doc_nodes_for_module_exports(
     &self,
+    parsed_source: &ParsedSource,
     module_decl: &ModuleDecl,
   ) -> Vec<DocNode> {
     match module_decl {
       ModuleDecl::ExportDecl(export_decl) => {
         vec![super::module::get_doc_node_for_export_decl(
           self,
+          parsed_source,
           export_decl,
         )]
       }
       ModuleDecl::ExportDefaultDecl(export_default_decl) => {
-        let (js_doc, location) =
-          self.details_for_span(export_default_decl.span);
+        let js_doc = js_doc_for_span(parsed_source, &export_default_decl.span);
+        let location = get_location(parsed_source, export_default_decl.span.lo);
         let name = "default".to_string();
 
         let doc_node = match &export_default_decl.decl {
           DefaultDecl::Class(class_expr) => {
-            let class_def =
-              crate::class::class_to_class_def(self, &class_expr.class);
+            let class_def = crate::class::class_to_class_def(
+              parsed_source,
+              &class_expr.class,
+            );
             DocNode::class(name, location, js_doc, class_def)
           }
           DefaultDecl::Fn(fn_expr) => {
             let function_def = crate::function::function_to_function_def(
-              self,
+              parsed_source,
               &fn_expr.function,
             );
             DocNode::function(name, location, js_doc, function_def)
@@ -322,7 +379,7 @@ impl DocParser {
           DefaultDecl::TsInterfaceDecl(interface_decl) => {
             let (_, interface_def) =
               crate::interface::get_doc_for_ts_interface_decl(
-                self,
+                parsed_source,
                 interface_decl,
               );
             DocNode::interface(name, location, js_doc, interface_def)
@@ -335,59 +392,67 @@ impl DocParser {
     }
   }
 
-  fn details_for_span(&self, span: Span) -> (Option<String>, Location) {
-    let js_doc = self.js_doc_for_span(span);
-    let location = self.ast_parser.get_span_location(span).into();
-    (js_doc, location)
-  }
-
-  pub fn get_doc_node_for_decl(&self, decl: &Decl) -> Option<DocNode> {
+  pub fn get_doc_node_for_decl(
+    &self,
+    parsed_source: &ParsedSource,
+    decl: &Decl,
+  ) -> Option<DocNode> {
     match decl {
       Decl::Class(class_decl) => {
         let (name, class_def) =
-          super::class::get_doc_for_class_decl(self, class_decl);
-        let (js_doc, location) = self.details_for_span(class_decl.class.span);
+          super::class::get_doc_for_class_decl(parsed_source, class_decl);
+        let js_doc = js_doc_for_span(parsed_source, &class_decl.class.span);
+        let location = get_location(parsed_source, class_decl.class.span.lo);
         Some(DocNode::class(name, location, js_doc, class_def))
       }
       Decl::Fn(fn_decl) => {
         let (name, function_def) =
-          super::function::get_doc_for_fn_decl(self, fn_decl);
-        let (js_doc, location) = self.details_for_span(fn_decl.function.span);
+          super::function::get_doc_for_fn_decl(parsed_source, fn_decl);
+        let js_doc = js_doc_for_span(parsed_source, &fn_decl.function.span);
+        let location = get_location(parsed_source, fn_decl.function.span.lo);
         Some(DocNode::function(name, location, js_doc, function_def))
       }
       Decl::Var(var_decl) => {
         let (name, var_def) = super::variable::get_doc_for_var_decl(var_decl);
-        let (js_doc, location) = self.details_for_span(var_decl.span);
+        let js_doc = js_doc_for_span(parsed_source, &var_decl.span);
+        let location = get_location(parsed_source, var_decl.span.lo);
         Some(DocNode::variable(name, location, js_doc, var_def))
       }
       Decl::TsInterface(ts_interface_decl) => {
         let (name, interface_def) =
           super::interface::get_doc_for_ts_interface_decl(
-            self,
+            parsed_source,
             ts_interface_decl,
           );
-        let (js_doc, location) = self.details_for_span(ts_interface_decl.span);
+        let js_doc = js_doc_for_span(parsed_source, &ts_interface_decl.span);
+        let location = get_location(parsed_source, ts_interface_decl.span.lo);
         Some(DocNode::interface(name, location, js_doc, interface_def))
       }
       Decl::TsTypeAlias(ts_type_alias) => {
         let (name, type_alias_def) =
           super::type_alias::get_doc_for_ts_type_alias_decl(
-            self,
+            parsed_source,
             ts_type_alias,
           );
-        let (js_doc, location) = self.details_for_span(ts_type_alias.span);
+        let js_doc = js_doc_for_span(parsed_source, &ts_type_alias.span);
+        let location = get_location(parsed_source, ts_type_alias.span.lo);
         Some(DocNode::type_alias(name, location, js_doc, type_alias_def))
       }
       Decl::TsEnum(ts_enum) => {
         let (name, enum_def) =
-          super::r#enum::get_doc_for_ts_enum_decl(self, ts_enum);
-        let (js_doc, location) = self.details_for_span(ts_enum.span);
+          super::r#enum::get_doc_for_ts_enum_decl(parsed_source, ts_enum);
+        let js_doc = js_doc_for_span(parsed_source, &ts_enum.span);
+        let location = get_location(parsed_source, ts_enum.span.lo);
         Some(DocNode::r#enum(name, location, js_doc, enum_def))
       }
       Decl::TsModule(ts_module) => {
-        let (name, namespace_def) =
-          super::namespace::get_doc_for_ts_module(self, ts_module);
-        let (js_doc, location) = self.details_for_span(ts_module.span);
+        let (name, namespace_def) = super::namespace::get_doc_for_ts_module(
+          self,
+          parsed_source,
+          ts_module,
+        );
+        let js_doc = js_doc_for_span(parsed_source, &ts_module.span);
+        let location = get_location(parsed_source, ts_module.span.lo);
         Some(DocNode::namespace(name, location, js_doc, namespace_def))
       }
     }
@@ -395,7 +460,7 @@ impl DocParser {
 
   fn get_imports_for_module_body(
     &self,
-    module_body: &[swc_ecmascript::ast::ModuleItem],
+    module_body: &[deno_ast::swc::ast::ModuleItem],
   ) -> HashMap<String, Import> {
     let mut imports = HashMap::new();
 
@@ -442,9 +507,9 @@ impl DocParser {
 
   pub fn get_reexports_for_module_body(
     &self,
-    module_body: Vec<swc_ecmascript::ast::ModuleItem>,
+    module_body: &[deno_ast::swc::ast::ModuleItem],
   ) -> Vec<node::Reexport> {
-    let imports = self.get_imports_for_module_body(&module_body);
+    let imports = self.get_imports_for_module_body(module_body);
 
     let mut reexports: Vec<node::Reexport> = vec![];
 
@@ -461,7 +526,7 @@ impl DocParser {
     }
 
     for node in module_body.iter() {
-      if let swc_ecmascript::ast::ModuleItem::ModuleDecl(module_decl) = node {
+      if let deno_ast::swc::ast::ModuleItem::ModuleDecl(module_decl) = node {
         let r = match module_decl {
           ModuleDecl::ExportNamed(named_export) => {
             if let Some(src) = &named_export.src {
@@ -556,16 +621,23 @@ impl DocParser {
 
   fn get_symbols_for_module_body(
     &self,
-    module_body: &[swc_ecmascript::ast::ModuleItem],
+    parsed_source: &ParsedSource,
+    module_body: &[deno_ast::swc::ast::ModuleItem],
   ) -> HashMap<String, DocNode> {
     let mut symbols = HashMap::new();
 
     for node in module_body.iter() {
       let doc_node = match node {
-        ModuleItem::Stmt(Stmt::Decl(decl)) => self.get_doc_node_for_decl(decl),
-        ModuleItem::ModuleDecl(ModuleDecl::ExportDecl(export_decl)) => Some(
-          super::module::get_doc_node_for_export_decl(self, export_decl),
-        ),
+        ModuleItem::Stmt(Stmt::Decl(decl)) => {
+          self.get_doc_node_for_decl(parsed_source, decl)
+        }
+        ModuleItem::ModuleDecl(ModuleDecl::ExportDecl(export_decl)) => {
+          Some(super::module::get_doc_node_for_export_decl(
+            self,
+            parsed_source,
+            export_decl,
+          ))
+        }
         _ => None,
       };
 
@@ -579,9 +651,10 @@ impl DocParser {
 
   pub fn get_doc_nodes_for_module_body(
     &self,
-    module_body: Vec<swc_ecmascript::ast::ModuleItem>,
+    parsed_source: &ParsedSource,
+    module_body: Vec<deno_ast::swc::ast::ModuleItem>,
   ) -> Vec<DocNode> {
-    let symbols = self.get_symbols_for_module_body(&module_body);
+    let symbols = self.get_symbols_for_module_body(parsed_source, &module_body);
 
     let mut doc_entries: Vec<DocNode> = Vec::new();
     let mut ambient_entries: Vec<DocNode> = Vec::new();
@@ -592,7 +665,9 @@ impl DocParser {
       match node {
         ModuleItem::Stmt(stmt) => {
           if let Stmt::Decl(decl) = stmt {
-            if let Some(doc_node) = self.get_doc_node_for_decl(decl) {
+            if let Some(doc_node) =
+              self.get_doc_node_for_decl(parsed_source, decl)
+            {
               let is_declared = self.get_declare_for_decl(decl);
               if self.private {
                 doc_entries.push(doc_node);
@@ -607,8 +682,9 @@ impl DocParser {
           // If it has imports/exports, it isn't ambient.
           is_ambient = false;
 
-          doc_entries
-            .extend(self.get_doc_nodes_for_module_exports(module_decl));
+          doc_entries.extend(
+            self.get_doc_nodes_for_module_exports(parsed_source, module_decl),
+          );
 
           match module_decl {
             ModuleDecl::ExportNamed(export_named) => {
@@ -645,14 +721,14 @@ impl DocParser {
                   });
                 }
               } else {
-                let (js_doc, location) =
-                  self.details_for_span(export_expr.span);
+                let js_doc = js_doc_for_span(parsed_source, &export_expr.span);
+                let location = get_location(parsed_source, export_expr.span.lo);
                 doc_entries.push(DocNode::variable(
                   String::from("default"),
                   location,
                   js_doc,
                   super::variable::VariableDef {
-                    kind: swc_ecmascript::ast::VarDeclKind::Var,
+                    kind: deno_ast::swc::ast::VarDeclKind::Var,
                     ts_type: super::ts_type::infer_simple_ts_type_from_expr(
                       export_expr.expr.as_ref(),
                       true,
@@ -674,39 +750,6 @@ impl DocParser {
     doc_entries
   }
 
-  pub fn js_doc_for_span(&self, span: Span) -> Option<String> {
-    let comments = self.ast_parser.get_span_comments(span);
-    let js_doc_comment = comments.iter().rev().find(|comment| {
-      comment.kind == CommentKind::Block && comment.text.starts_with('*')
-    })?;
-
-    let mut margin_pat = String::from("");
-    if let Some(margin) = self.ast_parser.source_map.span_to_margin(span) {
-      for _ in 0..margin {
-        margin_pat.push(' ');
-      }
-    }
-
-    let js_doc_re = Regex::new(r#" ?\* ?"#).unwrap();
-    let txt = js_doc_comment
-      .text
-      .split('\n')
-      .map(|line| js_doc_re.replace(line, "").to_string())
-      .map(|line| {
-        if line.starts_with(&margin_pat) {
-          line[margin_pat.len()..].to_string()
-        } else {
-          line
-        }
-      })
-      .collect::<Vec<String>>()
-      .join("\n");
-
-    let txt = txt.trim_start().trim_end().to_string();
-
-    Some(txt)
-  }
-
   fn get_declare_for_decl(&self, decl: &Decl) -> bool {
     match decl {
       Decl::Class(class_decl) => class_decl.declare,
